@@ -37,8 +37,9 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <gmodule.h>
+#include <glib/gi18n-lib.h>
 
-#include <gconf/gconf-client.h>
+#include <gst/gst.h>
 
 #include "brasero-media-private.h"
 
@@ -52,7 +53,8 @@
 #include "brasero-plugin-registration.h"
 #include "burn-caps.h"
 
-#define BRASERO_PLUGIN_PRIORITY_KEY			"/apps/brasero/config/priority"
+#define BRASERO_SCHEMA_PLUGINS				"org.gnome.brasero.plugins"
+#define BRASERO_PROPS_PRIORITY_KEY			"priority"
 
 typedef struct _BraseroPluginFlagPair BraseroPluginFlagPair;
 
@@ -88,18 +90,21 @@ struct _BraseroPluginConfOption {
 typedef struct _BraseroPluginPrivate BraseroPluginPrivate;
 struct _BraseroPluginPrivate
 {
+	GSettings *settings;
+
 	gboolean active;
 	guint group;
 
 	GSList *options;
 
-	gchar *error;
+	GSList *errors;
 
 	GType type;
 	gchar *path;
 	GModule *handle;
 
 	gchar *name;
+	gchar *display_name;
 	gchar *author;
 	gchar *description;
 	gchar *copyright;
@@ -139,6 +144,155 @@ enum
 static GTypeModuleClass* parent_class = NULL;
 static guint plugin_signals [LAST_SIGNAL] = { 0 };
 
+static void
+brasero_plugin_error_free (BraseroPluginError *error)
+{
+	g_free (error->detail);
+	g_free (error);
+}
+
+void
+brasero_plugin_add_error (BraseroPlugin *plugin,
+                          BraseroPluginErrorType type,
+                          const gchar *detail)
+{
+	BraseroPluginError *error;
+	BraseroPluginPrivate *priv;
+
+	g_return_if_fail (BRASERO_IS_PLUGIN (plugin));
+
+	priv = BRASERO_PLUGIN_PRIVATE (plugin);
+
+	error = g_new0 (BraseroPluginError, 1);
+	error->detail = g_strdup (detail);
+	error->type = type;
+
+	priv->errors = g_slist_prepend (priv->errors, error);
+}
+
+void
+brasero_plugin_test_gstreamer_plugin (BraseroPlugin *plugin,
+                                      const gchar *name)
+{
+	GstElement *element;
+
+	/* Let's see if we've got the plugins we need */
+	element = gst_element_factory_make (name, NULL);
+	if (!element)
+		brasero_plugin_add_error (plugin,
+		                          BRASERO_PLUGIN_ERROR_MISSING_GSTREAMER_PLUGIN,
+		                          name);
+	else
+		gst_object_unref (element);
+}
+
+void
+brasero_plugin_test_app (BraseroPlugin *plugin,
+                         const gchar *name,
+                         const gchar *version_arg,
+                         const gchar *version_format,
+                         gint version [3])
+{
+	gchar *standard_output = NULL;
+	gchar *standard_error = NULL;
+	guint major, minor, sub;
+	gchar *prog_path;
+	GPtrArray *argv;
+	gboolean res;
+	int i;
+
+	/* First see if this plugin can be used, i.e. if cdrecord is in
+	 * the path */
+	prog_path = g_find_program_in_path (name);
+	if (!prog_path) {
+		brasero_plugin_add_error (plugin,
+		                          BRASERO_PLUGIN_ERROR_MISSING_APP,
+		                          name);
+		return;
+	}
+
+	if (!g_file_test (prog_path, G_FILE_TEST_IS_EXECUTABLE)) {
+		g_free (prog_path);
+		brasero_plugin_add_error (plugin,
+		                          BRASERO_PLUGIN_ERROR_MISSING_APP,
+		                          name);
+		return;
+	}
+
+	/* make sure that's not a symlink pointing to something with another
+	 * name like wodim.
+	 * NOTE: we used to test the target and see if it had the same name as
+	 * the symlink with GIO. The problem is, when the symlink pointed to
+	 * another symlink, then GIO didn't follow that other symlink. And in
+	 * the end it didn't work. So forbid all symlink. */
+	if (g_file_test (prog_path, G_FILE_TEST_IS_SYMLINK)) {
+		brasero_plugin_add_error (plugin,
+		                          BRASERO_PLUGIN_ERROR_SYMBOLIC_LINK_APP,
+		                          name);
+		g_free (prog_path);
+		return;
+	}
+	/* Make sure it's a regular file */
+	else if (!g_file_test (prog_path, G_FILE_TEST_IS_REGULAR)) {
+		brasero_plugin_add_error (plugin,
+		                          BRASERO_PLUGIN_ERROR_MISSING_APP,
+		                          name);
+		g_free (prog_path);
+		return;
+	}
+
+	if (!version_arg) {
+		g_free (prog_path);
+		return;
+	}
+
+	/* Check version */
+	argv = g_ptr_array_new ();
+	g_ptr_array_add (argv, prog_path);
+	g_ptr_array_add (argv, (gchar *) version_arg);
+	g_ptr_array_add (argv, NULL);
+
+	res = g_spawn_sync (NULL,
+	                    (gchar **) argv->pdata,
+	                    NULL,
+	                    0,
+	                    NULL,
+	                    NULL,
+	                    &standard_output,
+	                    &standard_error,
+	                    NULL,
+	                    NULL);
+
+	g_ptr_array_free (argv, TRUE);
+	g_free (prog_path);
+
+	if (!res) {
+		brasero_plugin_add_error (plugin,
+		                          BRASERO_PLUGIN_ERROR_WRONG_APP_VERSION,
+		                          name);
+		return;
+	}
+
+	for (i = 0; i < 3 && version [i] >= 0; i++);
+
+	if ((standard_output && sscanf (standard_output, version_format, &major, &minor, &sub) == i)
+	||  (standard_error && sscanf (standard_error, version_format, &major, &minor, &sub) == i)) {
+		if (major < version [0]
+		||  (version [1] >= 0 && minor < version [1])
+		||  (version [2] >= 0 && sub < version [2]))
+			brasero_plugin_add_error (plugin,
+						  BRASERO_PLUGIN_ERROR_WRONG_APP_VERSION,
+						  name);
+	}
+	else
+		brasero_plugin_add_error (plugin,
+		                          BRASERO_PLUGIN_ERROR_WRONG_APP_VERSION,
+		                          name);
+
+	g_free (standard_output);
+	g_free (standard_error);
+}
+
 void
 brasero_plugin_set_compulsory (BraseroPlugin *self,
 			       gboolean compulsory)
@@ -167,10 +321,10 @@ brasero_plugin_set_active (BraseroPlugin *self, gboolean active)
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
 
-	was_active = brasero_plugin_get_active (self);
+	was_active = brasero_plugin_get_active (self, FALSE);
 	priv->active = active;
 
-	now_active = brasero_plugin_get_active (self);
+	now_active = brasero_plugin_get_active (self, FALSE);
 	if (was_active == now_active)
 		return;
 
@@ -185,17 +339,23 @@ brasero_plugin_set_active (BraseroPlugin *self, gboolean active)
 }
 
 gboolean
-brasero_plugin_get_active (BraseroPlugin *self)
+brasero_plugin_get_active (BraseroPlugin *plugin,
+                           gboolean ignore_errors)
 {
 	BraseroPluginPrivate *priv;
 
-	priv = BRASERO_PLUGIN_PRIVATE (self);
+	priv = BRASERO_PLUGIN_PRIVATE (plugin);
+
+	if (priv->type == G_TYPE_NONE)
+		return FALSE;
 
 	if (priv->priority < 0)
 		return FALSE;
 
-	if (priv->type == G_TYPE_NONE)
-		return FALSE;
+	if (priv->errors) {
+		if (!ignore_errors)
+			return FALSE;
+	}
 
 	return priv->active;
 }
@@ -217,10 +377,6 @@ brasero_plugin_cleanup_definition (BraseroPlugin *self)
 	priv->copyright = NULL;
 	g_free (priv->website);
 	priv->website = NULL;
-	if (priv->error) {
-		g_free (priv->error);
-		priv->error = NULL;
-	}
 }
 
 /**
@@ -405,47 +561,11 @@ brasero_plugin_conf_option_choice_get (BraseroPluginConfOption *option)
  * Used to set the caps of plugin
  */
 
-gchar *
-brasero_plugin_get_gconf_priority_key (BraseroPlugin *self)
-{
-	BraseroPluginPrivate *priv;
-	gchar *priority_path;
-	gchar *gconf_name;
-
-	gint type = 3;
-	BraseroBurnCaps *caps;
-	gchar *category [] = { "burn",
-			       "image",
-			       "convert",
-			       "misc" };
-
-	priv = BRASERO_PLUGIN_PRIVATE (self);
-
-	caps = brasero_burn_caps_get_default ();
-
-	/* make sure the name used has characters usable by GConf */
-	if (brasero_plugin_can_burn (self) == BRASERO_BURN_OK)
-		type = 0;
-	else if (brasero_plugin_can_image (self) == BRASERO_BURN_OK)
-		type = 1;
-	else if (brasero_plugin_can_convert (self) == BRASERO_BURN_OK)
-		type = 2;
-
-	gconf_name = gconf_escape_key (priv->name, -1);
-	priority_path = g_strdup_printf ("%s/%s-%s",
-					 BRASERO_PLUGIN_PRIORITY_KEY,
-					 gconf_name,
-					 category [type]);
-	g_free (gconf_name);
-	g_object_unref (caps);
-
-	return priority_path;
-}
-
 void
 brasero_plugin_define (BraseroPlugin *self,
 		       const gchar *name,
-		       const gchar *description,
+                       const gchar *display_name,
+                       const gchar *description,
 		       const gchar *author,
 		       guint priority)
 {
@@ -456,6 +576,7 @@ brasero_plugin_define (BraseroPlugin *self,
 	brasero_plugin_cleanup_definition (self);
 
 	priv->name = g_strdup (name);
+	priv->display_name = g_strdup (display_name);
 	priv->author = g_strdup (author);
 	priv->description = g_strdup (description);
 	priv->priority_original = priority;
@@ -480,13 +601,82 @@ brasero_plugin_get_group (BraseroPlugin *self)
 	return priv->group;
 }
 
-const gchar *
-brasero_plugin_get_error (BraseroPlugin *self)
+/**
+ * brasero_plugin_get_errors:
+ * @plugin: a #BraseroPlugin.
+ *
+ * This function returns a list of all errors that
+ * prevents the plugin from working properly.
+ *
+ * Returns: a #GSList of #BraseroPluginError structures or %NULL.
+ * It must not be freed.
+ **/
+
+GSList *
+brasero_plugin_get_errors (BraseroPlugin *plugin)
 {
 	BraseroPluginPrivate *priv;
 
-	priv = BRASERO_PLUGIN_PRIVATE (self);
-	return priv->error;
+	g_return_val_if_fail (BRASERO_IS_PLUGIN (plugin), NULL);
+	priv = BRASERO_PLUGIN_PRIVATE (plugin);
+	return priv->errors;
+}
+
+gchar *
+brasero_plugin_get_error_string (BraseroPlugin *plugin)
+{
+	gchar *error_string = NULL;
+	BraseroPluginPrivate *priv;
+	GString *string;
+	GSList *iter;
+
+	g_return_val_if_fail (BRASERO_IS_PLUGIN (plugin), NULL);
+
+	priv = BRASERO_PLUGIN_PRIVATE (plugin);
+
+	string = g_string_new (NULL);
+	for (iter = priv->errors; iter; iter = iter->next) {
+		BraseroPluginError *error;
+
+		error = iter->data;
+		switch (error->type) {
+			case BRASERO_PLUGIN_ERROR_MISSING_APP:
+				g_string_append_c (string, '\n');
+				g_string_append_printf (string, _("\"%s\" could not be found in the path"), error->detail);
+				break;
+			case BRASERO_PLUGIN_ERROR_MISSING_GSTREAMER_PLUGIN:
+				g_string_append_c (string, '\n');
+				g_string_append_printf (string, _("\"%s\" GStreamer plugin could not be found"), error->detail);
+				break;
+			case BRASERO_PLUGIN_ERROR_WRONG_APP_VERSION:
+				g_string_append_c (string, '\n');
+				g_string_append_printf (string, _("The version of \"%s\" is too old"), error->detail);
+				break;
+			case BRASERO_PLUGIN_ERROR_SYMBOLIC_LINK_APP:
+				g_string_append_c (string, '\n');
+				g_string_append_printf (string, _("\"%s\" is a symbolic link pointing to another program"), error->detail);
+				break;
+			case BRASERO_PLUGIN_ERROR_MISSING_LIBRARY:
+				g_string_append_c (string, '\n');
+				g_string_append_printf (string, _("\"%s\" could not be found"), error->detail);
+				break;
+			case BRASERO_PLUGIN_ERROR_LIBRARY_VERSION:
+				g_string_append_c (string, '\n');
+				g_string_append_printf (string, _("The version of \"%s\" is too old"), error->detail);
+				break;
+			case BRASERO_PLUGIN_ERROR_MODULE:
+				g_string_append_c (string, '\n');
+				g_string_append (string, error->detail);
+				break;
+
+			default:
+				break;
+		}
+	}
+
+	error_string = string->str;
+	g_string_free (string, FALSE);
+	return error_string;
 }
 
 static BraseroPluginFlags *
@@ -582,7 +772,6 @@ brasero_plugin_get_all_flags (GSList *flags_list,
 
 	flags = brasero_plugin_get_flags (flags_list, media);
 	if (!flags) {
-
 		if (supported_retval)
 			*supported_retval = BRASERO_BURN_FLAG_NONE;
 		if (compulsory_retval)
@@ -843,6 +1032,16 @@ brasero_plugin_get_name (BraseroPlugin *plugin)
 	return priv->name;
 }
 
+const gchar *
+brasero_plugin_get_display_name (BraseroPlugin *plugin)
+{
+	BraseroPluginPrivate *priv;
+
+	priv = BRASERO_PLUGIN_PRIVATE (plugin);
+	return priv->display_name ? priv->display_name:priv->name;
+
+}
+
 const gchar *
 brasero_plugin_get_author (BraseroPlugin *plugin)
 {
@@ -907,6 +1106,10 @@ brasero_plugin_get_gtype (BraseroPlugin *self)
 	BraseroPluginPrivate *priv;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
+
+	if (priv->errors)
+		return G_TYPE_NONE;
+
 	return priv->type;
 }
 
@@ -930,7 +1133,6 @@ brasero_plugin_unload (GTypeModule *module)
 static gboolean
 brasero_plugin_load_real (BraseroPlugin *plugin) 
 {
-	gchar *error = NULL;
 	BraseroPluginPrivate *priv;
 	BraseroPluginRegisterType register_func;
 
@@ -944,7 +1146,8 @@ brasero_plugin_load_real (BraseroPlugin *plugin)
 
 	priv->handle = g_module_open (priv->path, G_MODULE_BIND_LAZY);
 	if (!priv->handle) {
-		priv->error = g_strdup (g_module_error ());
+		brasero_plugin_add_error (plugin, BRASERO_PLUGIN_ERROR_MODULE, g_module_error ());
+		BRASERO_BURN_LOG ("Module %s can't be loaded: g_module_open failed ()", priv->name);
 		return FALSE;
 	}
 
@@ -954,13 +1157,7 @@ brasero_plugin_load_real (BraseroPlugin *plugin)
 		return FALSE;
 	}
 
-	priv->type = register_func (plugin, &error);
-	if (error) {
-		if (priv->error)
-			g_free (priv->error);
-		priv->error = error;
-	}
-
+	priv->type = register_func (plugin);
 	brasero_burn_debug_setup_module (priv->handle);
 	return TRUE;
 }
@@ -978,106 +1175,128 @@ brasero_plugin_load (GTypeModule *module)
 }
 
 static void
-brasero_plugin_priority_changed (GConfClient *client,
-				 guint cxn,
-				 GConfEntry *entry,
-				 gpointer data)
+brasero_plugin_priority_changed (GSettings *settings,
+                                 const gchar *key,
+                                 BraseroPlugin *self)
 {
 	BraseroPluginPrivate *priv;
-	BraseroPlugin *self;
 	gboolean is_active;
-	GConfValue *value;
 
-	self = BRASERO_PLUGIN (data);
 	priv = BRASERO_PLUGIN_PRIVATE (self);
 
-	value = gconf_entry_get_value (entry);
-	if (!value)
-		priv->priority = 0;
-	else if (value->type != GCONF_VALUE_INT)
-		return;
-	else
-		priv->priority = gconf_value_get_int (value);
+	/* At the moment it can only be the priority key */
+	priv->priority = g_settings_get_int (settings, BRASERO_PROPS_PRIORITY_KEY);
 
-	is_active = brasero_plugin_get_active (self);
+	is_active = brasero_plugin_get_active (self, FALSE);
 
 	g_object_notify (G_OBJECT (self), "priority");
-	if (is_active != brasero_plugin_get_active (self))
+	if (is_active != brasero_plugin_get_active (self, FALSE))
 		g_signal_emit (self,
 			       plugin_signals [ACTIVATED_SIGNAL],
 			       0,
-			       brasero_plugin_get_active (self));
+			       is_active);
+}
+
+typedef void	(* BraseroPluginCheckConfig)	(BraseroPlugin *plugin);
+
+/**
+ * brasero_plugin_check_plugin_ready:
+ * @plugin: a #BraseroPlugin.
+ *
+ * Ask a plugin to check whether it can operate.
+ * brasero_plugin_can_operate () should be called
+ * afterwards to know whether it can operate or not.
+ *
+ **/
+void
+brasero_plugin_check_plugin_ready (BraseroPlugin *plugin)
+{
+	GModule *handle;
+	BraseroPluginPrivate *priv;
+	BraseroPluginCheckConfig function = NULL;
+
+	g_return_if_fail (BRASERO_IS_PLUGIN (plugin));
+	priv = BRASERO_PLUGIN_PRIVATE (plugin);
+
+	if (priv->errors) {
+		g_slist_foreach (priv->errors, (GFunc) brasero_plugin_error_free, NULL);
+		g_slist_free (priv->errors);
+		priv->errors = NULL;
+	}
+
+	handle = g_module_open (priv->path, 0);
+	if (!handle) {
+		brasero_plugin_add_error (plugin, BRASERO_PLUGIN_ERROR_MODULE, g_module_error ());
+		BRASERO_BURN_LOG ("Module %s can't be loaded: g_module_open failed ()", priv->name);
+		return;
+	}
+
+	if (!g_module_symbol (handle, "brasero_plugin_check_config", (gpointer) &function)) {
+		g_module_close (handle);
+		BRASERO_BURN_LOG ("Module %s has no check config function", priv->name);
+		return;
+	}
+
+	function (BRASERO_PLUGIN (plugin));
+	g_module_close (handle);
 }
 
 static void
 brasero_plugin_init_real (BraseroPlugin *object)
 {
 	GModule *handle;
-	GConfValue *value;
-	gint priority_gconf;
-	GConfClient *client;
-	gchar *priority_path;
+	gchar *settings_path;
 	BraseroPluginPrivate *priv;
-	BraseroPluginRegisterType function;
+	BraseroPluginRegisterType function = NULL;
 
 	priv = BRASERO_PLUGIN_PRIVATE (object);
 
 	g_type_module_set_name (G_TYPE_MODULE (object), priv->name);
 
-	handle = g_module_open (priv->name, 0);
+	handle = g_module_open (priv->path, 0);
 	if (!handle) {
-		BRASERO_BURN_LOG ("Module can't be loaded: g_module_open failed");
+		brasero_plugin_add_error (object, BRASERO_PLUGIN_ERROR_MODULE, g_module_error ());
+		BRASERO_BURN_LOG ("Module %s (at %s) can't be loaded: g_module_open failed ()", priv->name, priv->path);
 		return;
 	}
 
 	if (!g_module_symbol (handle, "brasero_plugin_register", (gpointer) &function)) {
 		g_module_close (handle);
-		BRASERO_BURN_LOG ("Module can't be loaded: no register function");
+		BRASERO_BURN_LOG ("Module %s can't be loaded: no register function, priv->name", priv->name);
 		return;
 	}
 
-	priv->type = function (BRASERO_PLUGIN (object), &priv->error);
+	priv->type = function (object);
 	if (priv->type == G_TYPE_NONE) {
 		g_module_close (handle);
-		BRASERO_BURN_LOG ("Module encountered an error while registering its capabilities:\n%s",
-				  priv->error ? priv->error:"unknown error");
+		BRASERO_BURN_LOG ("Module %s encountered an error while registering its capabilities", priv->name);
 		return;
 	}
 
-	BRASERO_BURN_LOG ("Module %s successfully loaded", priv->name);
-	g_module_close (handle);
-
 	/* now see if we need to override the hardcoded priority of the plugin */
-	client = gconf_client_get_default ();
-	priority_path = brasero_plugin_get_gconf_priority_key (object);
+	settings_path = g_strconcat ("/apps/brasero/plugins/",
+	                             priv->name,
+	                             G_DIR_SEPARATOR_S,
+	                             NULL);
+	priv->settings = g_settings_new_with_path (BRASERO_SCHEMA_PLUGINS,
+	                                           settings_path);
+	g_free (settings_path);
 
-	/* get the gconf priority */
-	value = gconf_client_get (client, priority_path, NULL);
-	if (value) {
-		priority_gconf = gconf_value_get_int (value);
-		priv->priority = priority_gconf;
-		gconf_value_free (value);
-	}
-	else {
-		/* set a default for an advanced user to modify */
-		BRASERO_BURN_LOG ("Creating default priority key %s", priority_path);
-		gconf_client_set_int (client,
-				      priority_path,
-				      0,
-				      NULL);
-	}
+	priv->priority = g_settings_get_int (priv->settings, BRASERO_PROPS_PRIORITY_KEY);
 
-	/* get notifications when priority changes */
-	priv->notify_priority = gconf_client_notify_add (client,
-							 priority_path,
-							 brasero_plugin_priority_changed,
-							 object,
-							 NULL,
-							 NULL);
+	/* Make sure a key is created for each plugin */
+	if (!priv->priority)
+		g_settings_set_int (priv->settings, BRASERO_PROPS_PRIORITY_KEY, 0);
 
-	/* No need to emit notify:: here */
-	g_free (priority_path);
-	g_object_unref (client);
+	g_signal_connect (priv->settings,
+	                  "changed",
+	                  G_CALLBACK (brasero_plugin_priority_changed),
+	                  object);
+
+	/* Check if it can operate */
+	brasero_plugin_check_plugin_ready (object);
+
+	g_module_close (handle);
 }
 
 static void
@@ -1104,6 +1323,7 @@ brasero_plugin_finalize (GObject *object)
 	}
 
 	g_free (priv->name);
+	g_free (priv->display_name);
 	g_free (priv->author);
 	g_free (priv->description);
 
@@ -1113,18 +1333,15 @@ brasero_plugin_finalize (GObject *object)
 	g_slist_foreach (priv->blank_flags, (GFunc) g_free, NULL);
 	g_slist_free (priv->blank_flags);
 
-	if (priv->notify_priority) {
-		GConfClient *client;
-
-		client = gconf_client_get_default ();
-		gconf_client_notify_remove (client, priv->notify_priority);
-		priv->notify_priority = 0;
-		g_object_unref (client);
+	if (priv->settings) {
+		g_object_unref (priv->settings);
+		priv->settings = NULL;
 	}
 
-	if (priv->error) {
-		g_free (priv->error);
-		priv->error = NULL;
+	if (priv->errors) {
+		g_slist_foreach (priv->errors, (GFunc) brasero_plugin_error_free, NULL);
+		g_slist_free (priv->errors);
+		priv->errors = NULL;
 	}
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -1209,7 +1426,6 @@ brasero_plugin_class_init (BraseroPluginClass *klass)
 	object_class->set_property = brasero_plugin_set_property;
 	object_class->get_property = brasero_plugin_get_property;
 
-	parent_class = G_TYPE_MODULE_CLASS (g_type_class_peek_parent (klass));
 	module_class->load = brasero_plugin_load;
 	module_class->unload = brasero_plugin_unload;
 
