@@ -1480,6 +1480,11 @@ brasero_medium_get_page_2A_write_speed_desc (BraseroMedium *self,
 
 	desc = page_2A->wr_spd_desc;
 	for (i = 0; i < desc_num; i ++) {
+		/* It happens (I have such a drive) that it returns descriptors
+		 * with the same speeds each (in this case the maximum) */
+		if (i > 0 && priv->wr_speeds [i-1] == BRASERO_GET_16 (desc [i].speed))
+			continue;
+
 		priv->wr_speeds [i] = BRASERO_GET_16 (desc [i].speed);
 		max_wrt = MAX (max_wrt, priv->wr_speeds [i]);
 	}
@@ -2786,10 +2791,19 @@ brasero_medium_read_CD_TEXT (BraseroMedium *self,
 		return;
 	}
 
-	/* Get the number of CD-Text Data Packs */
-	num = (BRASERO_GET_16 (cd_text->hdr->len) -
-	      (sizeof (BraseroScsiTocPmaAtipHdr) - sizeof (cd_text->hdr->len)))  /
-	       sizeof (BraseroScsiCDTextPackData);
+	/* Get the number of CD-Text Data Packs.
+	 * Some drives seem to report an idiotic cd_text->hdr->len. So use size
+	 * to be on a safer side. */
+	if (size < sizeof (BraseroScsiTocPmaAtipHdr)) {
+		g_free (cd_text);
+		return;
+	}
+
+	num = (size - sizeof (BraseroScsiTocPmaAtipHdr)) / sizeof (BraseroScsiCDTextPackData);
+	if (num <= 0) {
+		g_free (cd_text);
+		return;
+	}
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
